function llfn = bgllfn(bigthet)
% Uses the Kalman filter to evaluate the negative log likelihood
%   function for the Barro-Gordon model when the natural rate of
%   unemployment follows an AR(1) in first differences.
%
% THIS PROGRAM WAS WRITTEN FOR MATLAB BY
%
%   PETER N. IRELAND
%   BOSTON COLLEGE
%   DEPARTMENT OF ECONOMICS
%   140 COMMONWEALTH AVENUE
%   CHESTNUT HILL, MA 02467-3806
%   irelandp@bc.edu
%
%  FINANCIAL SUPPORT FROM THE NATIONAL SCIENCE FOUNDATION UNDER
%    GRANT NO. SES-9985763 IS GRATEFULLY ACKNOWLEDGED.

% define variables

  global ut pit

  T = length(ut) - 1;

  alpha = bigthet(1); 
  biga = bigthet(2);
  lamtr = bigthet(3);
  chole = bigthet(4);
  choln = bigthet(5);
  cholen = bigthet(6);

  lam = lamtr/(1+abs(lamtr));

% construct state-space model

  yt = [ pit(2:T+1)-alpha*biga*ut(2:T+1) ut(2:T+1)-ut(1:T) ];

  CHOLSIG = [ chole 0 ; cholen choln ];

  SIG = CHOLSIG*CHOLSIG';

  F = [ lam 0 0 0 ; 0 0 0 0 ; 0 0 0 0 ; 0 0 1 0 ];

  Q = [ 1 0 ; 1 0 ; 0 1 ; 0 0 ];

  H = [ 0 -alpha*biga 1+alpha^2*biga 0 ; 1 0 -alpha alpha ];

  QSQ = Q*SIG*Q';

% initialize Kalman filter

  xit = zeros(4,1);

  Pt = inv(eye(16)-kron(F,F))*QSQ(:);

  Pt = reshape(Pt,4,4);

% evaluate negative log likelihood function

  llfn =  T*log(2*pi);

  for t = 1:T

    et = yt(t,:)' - H*xit;

    llfn = llfn + (1/2)*log(det(H*Pt*H')) + (1/2)*et'*inv(H*Pt*H')*et;

    Kt = F*Pt*H'*inv(H*Pt*H');

    xit = F*xit + Kt*et;

    Pt = (F-Kt*H)*Pt*(F'-H'*Kt') + QSQ;

  end
