% philou.m: Performs Phillips-Ouliaris test for cointegration,
%            with or without a constant term in the initial
%            cointegrating regression.
%
% THIS PROGRAM WAS WRITTEN FOR MATLAB BY
%
%   PETER N. IRELAND
%   BOSTON COLLEGE
%   DEPARTMENT OF ECONOMICS
%   140 COMMONWEALTH AVENUE
%   CHESTNUT HILL, MA 02467-3806
%   irelandp@bc.edu
%
%  FINANCIAL SUPPORT FROM THE NATIONAL SCIENCE FOUNDATION UNDER
%    GRANT NO. SES-9985763 IS GRATEFULLY ACKNOWLEDGED.

% load data

  load def.dat
  yt = def;

  load u.dat;
  xt = u;  

  T = length(yt);

% adjust sample size if necessary

% yt = yt(41:T);

% xt = xt(41:T);

% T = length(yt);

% include constant

%  xt = [ ones(T,1) xt ];

% perform conintegrating regression and obtain residuals

  betvec = inv(xt'*xt)*xt'*yt;

  ut = yt - xt*betvec;

% perform residual regression and compute preliminary statistics

  yt = ut(2:T);

  xt = ut(1:T-1);

  rho = inv(xt'*xt)*xt'*yt;

  et = yt - xt*rho;

  s2 = et'*et/(T-2);

  s = sqrt(s2);

  sigr = s*sqrt(inv(xt'*xt));

  tstat = (rho-1)/sigr;

% compute Newey-West variance

  gamo = (1/(T-1))*et'*et;

  gamvec = zeros(50,1);

  for j = 1:51

    gamet = et(j+1:T-1);

    gamej = ut(1:T-j-1);

    gamvec(j) = (1/(T-1))*gamet'*gamej;

  end

% compute Phillips-Ouliaris statistic for various values of q

  postat = zeros(51,1);

  postat(1) = tstat;

  for q = 1:50

    lam2 = gamo;

    for j = 1:q

      lam2 = lam2 + 2*(1-j/(q+1))*gamvec(j);

    end

    lam = sqrt(lam2);

    postat(q+1) = (sqrt(gamo/lam2))*tstat ...
                    - (1/2)*((lam2-gamo)/lam)*((T-1)*sigr/s);

  end

% compute Andrews' optimal q

  pihat = inv(et(1:T-2)'*et(1:T-2))*et(1:T-2)'*et(2:T-1);

  qstar = 4*pihat^2/((1-pihat)^2*(1+pihat)^2);
